import * as React from 'react';
import { ComponentPropTypes } from './component-common';
import { Props as InputControlProps } from './controls/input';
import { Props as InputGroupProps } from './input-group';
declare type InputControlPropsCleaned = Omit<InputControlProps, 'id' | 'name' | 'className'>;
declare type SupportedInputTypes = 'color' | 'date' | 'datetime' | 'datetime-local' | 'email' | 'hidden' | 'month' | 'number' | 'password' | 'range' | 'search' | 'tel' | 'text' | 'time' | 'url' | 'week';
interface Props extends ComponentPropTypes, InputControlPropsCleaned, InputGroupProps {
    type: SupportedInputTypes;
    value: string;
    blurDebounceInterval: number;
    changeDebounceInterval: number;
    updateOnBlur: boolean;
    updateOnChange: boolean;
    required: boolean;
    blurCallback: (name: any, value: any) => {};
    keyDownCallback: (event: any) => {};
}
interface State {
    value: string;
}
declare class Input extends React.Component<Props, State> {
    private element;
    private changeDebounced;
    private blurDebounced;
    static defaultProps: {
        type: string;
        value: string;
        updateOnBlur: boolean;
        updateOnChange: boolean;
        blurDebounceInterval: number;
        changeDebounceInterval: number;
        blurCallback: () => void;
        keyDownCallback: () => void;
        required: boolean;
        changeCallback: (name: string, value: any) => void;
        onSetValue: (value: any) => void;
        errorMessages: never[];
        help: string;
        label: null;
        layout: "horizontal" | "vertical" | "elementOnly";
        showErrors: boolean;
        rowClassName: string;
        labelClassName: string;
        elementWrapperClassName: string;
        isPristine: () => boolean;
    } | {
        type: string;
        value: string;
        updateOnBlur: boolean;
        updateOnChange: boolean;
        blurDebounceInterval: number;
        changeDebounceInterval: number;
        blurCallback: () => void;
        keyDownCallback: () => void;
        required: boolean;
        addonAfter?: React.ReactNode;
        addonBefore?: React.ReactNode;
        buttonAfter?: React.ReactNode;
        buttonBefore?: React.ReactNode;
        changeCallback: (name: string, value: any) => void;
        onSetValue: (value: any) => void;
        errorMessages: never[];
        help: string;
        label: null;
        layout: "horizontal" | "vertical" | "elementOnly";
        showErrors: boolean;
        rowClassName: string;
        labelClassName: string;
        elementWrapperClassName: string;
        isPristine: () => boolean;
    };
    constructor(props: any);
    componentWillReceiveProps: (nextProps: any) => void;
    private handleChange;
    private handleBlur;
    private handleKeyDown;
    private initElementRef;
    render(): JSX.Element;
}
export default Input;
