var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import debounce from '../debounce';
import { ComponentPropKeys, componentDefaultProps, } from './component-common';
import ErrorMessages from './error-messages';
import Help from './help';
import InputControl from './controls/input';
import InputGroup from './input-group';
import Row from './row';
var defaultProps = __assign({}, componentDefaultProps, InputGroup.defaultProps, { type: 'text', value: '', updateOnBlur: true, updateOnChange: true, blurDebounceInterval: 0, changeDebounceInterval: 500, blurCallback: function () { }, keyDownCallback: function () { }, required: false });
var Input = /** @class */ (function (_super) {
    __extends(Input, _super);
    function Input(props) {
        var _this = _super.call(this, props) || this;
        _this.componentWillReceiveProps = function (nextProps) {
            var nextValue = nextProps.value;
            var stateValue = _this.state.value;
            var onSetValue = _this.props.onSetValue;
            var isValueChanging = nextValue !== stateValue;
            if (isValueChanging) {
                _this.setState({ value: nextValue });
                onSetValue(nextValue);
            }
        };
        _this.handleChange = function (event) {
            var value = event.currentTarget.value;
            var _a = _this.props, updateOnChange = _a.updateOnChange, changeCallback = _a.changeCallback, name = _a.name;
            _this.setState({ value: value });
            if (updateOnChange) {
                _this.changeDebounced(value);
            }
            changeCallback(name, value);
        };
        _this.handleBlur = function (event) {
            var value = event.currentTarget.value;
            var _a = _this.props, updateOnBlur = _a.updateOnBlur, isPristine = _a.isPristine, blurCallback = _a.blurCallback, name = _a.name, propValue = _a.value;
            _this.setState({ value: value });
            if (updateOnBlur) {
                _this.changeDebounced.cancel();
                if (isPristine()) {
                    // should update as we have just left a pristine input
                    _this.blurDebounced(value);
                }
                else if (propValue !== value) {
                    // should update because the value has changed
                    _this.blurDebounced(value);
                }
            }
            blurCallback(name, value);
        };
        _this.handleKeyDown = function (event) {
            var keyDownCallback = _this.props.keyDownCallback;
            if (event.key === 'Enter') {
                _this.changeDebounced.flush();
            }
            keyDownCallback(event);
        };
        _this.initElementRef = function (control) {
            _this.element = control ? control.element : null;
        };
        var value = props.value, onSetValue = props.onSetValue, changeDebounceInterval = props.changeDebounceInterval, blurDebounceInterval = props.blurDebounceInterval;
        _this.state = { value: value };
        _this.changeDebounced = debounce(onSetValue, changeDebounceInterval);
        _this.blurDebounced = debounce(onSetValue, blurDebounceInterval);
        return _this;
    }
    Input.prototype.render = function () {
        var inputProps = __assign({}, this.props);
        ComponentPropKeys.forEach(function (key) {
            delete inputProps[key];
        });
        delete inputProps.addonAfter;
        delete inputProps.addonBefore;
        delete inputProps.buttonAfter;
        delete inputProps.buttonBefore;
        delete inputProps.blurDebounceInterval;
        delete inputProps.changeDebounceInterval;
        delete inputProps.updateOnBlur;
        delete inputProps.updateOnChange;
        delete inputProps.value;
        delete inputProps.keyDownCallback;
        delete inputProps.blurCallback;
        var value = this.state.value;
        var _a = this.props, addonAfter = _a.addonAfter, addonBefore = _a.addonBefore, buttonAfter = _a.buttonAfter, buttonBefore = _a.buttonBefore, errorMessages = _a.errorMessages, required = _a.required, help = _a.help, id = _a.id, layout = _a.layout, showErrors = _a.showErrors, type = _a.type;
        var markAsInvalid = showErrors && (errorMessages.length > 0 || required);
        var control = (React.createElement(InputControl, __assign({}, inputProps, { value: value, className: markAsInvalid ? 'is-invalid' : '', onChange: this.handleChange, onBlur: this.handleBlur, onKeyDown: this.handleKeyDown, ref: this.initElementRef })));
        if (type === 'hidden') {
            return control;
        }
        if (addonBefore || addonAfter || buttonBefore || buttonAfter) {
            control = React.createElement(InputGroup, __assign({}, this.props), control);
        }
        if (layout === 'elementOnly') {
            return control;
        }
        return (React.createElement(Row, __assign({}, this.props, { htmlFor: id }),
            React.createElement(React.Fragment, null,
                control,
                help ? React.createElement(Help, { help: help }) : null,
                showErrors ? React.createElement(ErrorMessages, { messages: errorMessages }) : null)));
    };
    Input.defaultProps = defaultProps;
    return Input;
}(React.Component));
export default Input;
//# sourceMappingURL=input.js.map