var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import { withFormsy } from 'formsy-react';
import FrcContext from '../context/frc';
var getDisplayName = function (component) {
    return component.displayName ||
        component.name ||
        (typeof component === 'string' ? component : 'Component');
};
/**
 * Component HOC
 * -------------
 *
 * This HOC provides shared code for our form components.
 *
 * We use this to merge props set using our OptionsProvider, so that
 * we can set commonly used props on an enclosing component.
 *
 * This allows us to set these properties 'as a whole' for each component in the
 * the form, while retaining the ability to override the prop on a per-component
 * basis.
 */
var FormsyReactComponent = function (ComposedComponent) {
    var ComponentHOC = /** @class */ (function (_super) {
        __extends(ComponentHOC, _super);
        function ComponentHOC() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.getBooleanFromPropsAndContext = function (propsValue, contextValue, defaultValue) {
                if (typeof propsValue === 'boolean') {
                    return propsValue;
                }
                if (typeof contextValue === 'boolean') {
                    return contextValue;
                }
                return defaultValue;
            };
            /**
             * getId
             * -----
             *
             * The ID is used as an attribute on the form control, and is used to allow
             * associating the label element with the form control.
             *
             * If we don't explicitly pass an `id` prop, we generate one based on the
             * `name` and `label` properties.
             */
            _this.getId = function () {
                var _a = _this.props, id = _a.id, label = _a.label, name = _a.name;
                if (id !== '') {
                    return id;
                }
                return [
                    'frc',
                    name
                        .split('[')
                        .join('_')
                        .replace(']', ''),
                    _this.hashString(JSON.stringify(label)),
                ].join('-');
            };
            _this.hashString = function (inString) {
                var hash = 0;
                for (var i = 0; i < inString.length; i += 1) {
                    // eslint-disable-next-line no-bitwise
                    hash = ((hash << 5) - hash + inString.charCodeAt(i)) & 0xffffffff;
                }
                return hash;
            };
            // Determine whether to show errors, or not.
            _this.shouldShowErrors = function (validatePristine, validateBeforeSubmit) {
                var _a = _this.props, isPristine = _a.isPristine, isFormSubmitted = _a.isFormSubmitted, isValid = _a.isValid;
                if (isPristine() === true) {
                    if (validatePristine === false) {
                        return false;
                    }
                }
                if (validateBeforeSubmit === false) {
                    if (isFormSubmitted() === false) {
                        return false;
                    }
                }
                return isValid() === false;
            };
            return _this;
        }
        // We pass through all unknown props, but delete some formsy HOC props
        // that we know we don't need.
        ComponentHOC.prototype.render = function () {
            var _this = this;
            return (React.createElement(FrcContext.Consumer, null, function (context) {
                var _a = _this.props, componentRef = _a.componentRef, disabled = _a.disabled, getErrorMessages = _a.getErrorMessages, getValue = _a.getValue, isFormDisabled = _a.isFormDisabled, isPristine = _a.isPristine, isRequired = _a.isRequired, layout = _a.layout, setValue = _a.setValue, propValidateBeforeSubmit = _a.validateBeforeSubmit, propValidatePristine = _a.validatePristine, elementWrapperClassName = _a.elementWrapperClassName, labelClassName = _a.labelClassName, rowClassName = _a.rowClassName;
                var contextLayout = context.layout, contextValidateBeforeSubmit = context.validateBeforeSubmit, contextValidatePristine = context.validatePristine, contextElementWrapperClassName = context.elementWrapperClassName, contextLabelClassName = context.labelClassName, contextRowClassName = context.rowClassName;
                var validatePristine = _this.getBooleanFromPropsAndContext(propValidatePristine, contextValidatePristine, false);
                var validateBeforeSubmit = _this.getBooleanFromPropsAndContext(propValidateBeforeSubmit, contextValidateBeforeSubmit, true);
                /**
                 * For ClassNames:
                 *
                 * - elementWrapperClassName
                 * - labelClassName
                 * - rowClassName
                 *
                 * We combine the parent context value with the component prop
                 * value, the value from props comes second, so that it may override
                 * the value set in the context. This is used for CSS classnames,
                 * where the value is finally parsed using `JedWatson/classnames`.
                 */
                var props = __assign({}, _this.props, { elementWrapperClassName: [
                        contextElementWrapperClassName,
                        elementWrapperClassName,
                    ], labelClassName: [contextLabelClassName, labelClassName], rowClassName: [contextRowClassName, rowClassName], disabled: isFormDisabled() || disabled, errorMessages: getErrorMessages(), id: _this.getId(), isPristine: isPristine, layout: layout || contextLayout, ref: componentRef, required: isRequired(), showErrors: _this.shouldShowErrors(validatePristine, validateBeforeSubmit), value: getValue(), onSetValue: setValue });
                // Props that we don't need to pass to our composed components.
                var unusedPropNames = [
                    // From formsy-react HOC...
                    'getErrorMessage',
                    'getErrorMessages',
                    'getValue',
                    'hasValue',
                    'isFormDisabled',
                    'isFormSubmitted',
                    'isRequired',
                    'isValid',
                    'isValidValue',
                    'resetValue',
                    'setValidations',
                    'setValue',
                    'showError',
                    'showRequired',
                    'validationError',
                    'validationErrors',
                    'validations',
                    'innerRef',
                    // From formsy-react-component HOC...
                    'componentRef',
                    'validateBeforeSubmit',
                    'validatePristine',
                ];
                unusedPropNames.forEach(function (propName) {
                    delete props[propName];
                });
                return React.createElement(ComposedComponent, __assign({}, props));
            }));
        };
        ComponentHOC.displayName = "withFRC(" + getDisplayName(ComposedComponent) + ")";
        // TODO: Should we add default props for the following?:
        // * elementWrapperClassName
        // * labelClassName
        // * rowClassName
        // The following props get their default values from the parent context.
        // * layout
        // * validatePristine: (See getValidatePristine, defaults to 'false'),
        // * validateBeforeSubmit: (See getValidateOnSubmit, defaults to 'false'),
        ComponentHOC.defaultProps = {
            disabled: false,
            help: '',
            id: '',
            label: '',
            layout: '',
            validateBeforeSubmit: null,
            validatePristine: null,
        };
        return ComponentHOC;
    }(React.Component));
    return withFormsy(ComponentHOC);
};
export default FormsyReactComponent;
//# sourceMappingURL=component.js.map